import {
  ActionSheet,
  Button,
  Cell,
  CountDown,
  Icon,
  Image,
  Popup,
  Swipe,
  SwipeItem,
  Tag
} from 'vant'
import { defineComponent, onMounted, reactive, ref } from 'vue'
import { useRoute, useRouter } from 'vue-router'
import styles from './index.module.less'
import iconQuestionNums from '../images/icon-question-nums.png'
import iconCountDown from '../images/icon-count-down.png'
import iconButtonList from '../images/icon-button-list.png'
import OSticky from '@/components/o-sticky'
import ChoiceQuestion from '../model/choice-question'
import AnswerList from '../model/answer-list'
import ODialog from '@/components/o-dialog'
import DragQuestion from '../model/drag-question'
import KeepLookQuestion from '../model/keep-look-question'
import PlayQuestion from '../model/play-question'
import request from '@/helpers/request'
import dayjs from 'dayjs'
import ResultFinish from '../model/result-finish'

export const QuestionType = {
  RADIO: 'RADIO',
  CHECKBOX: 'CHECKBOX',
  LINK: 'LINK',
  SORT: 'SORT',
  PLAY: 'PLAY'
}

export default defineComponent({
  name: 'unit-detail',
  setup() {
    const route = useRoute()
    const router = useRouter()
    const countDownRef = ref()
    const swipeRef = ref()
    const state = reactive({
      id: route.query.id,
      examDetail: {} as any,
      visiableAnswer: false,
      currentIndex: 0,
      questionList: [],
      time: 0,
      visiableSure: false,
      visiableResult: false,
      resultInfo: {} as any,
      resultStatusType: 'SUCCESS', // 'SUCCESS' | 'FAIL'
      visiableExam: false, // 考试已结束
      nextStatus: false
    })

    const getExamDetails = async () => {
      try {
        const { data } = await request.post(
          '/api-student/studentUnitExamination/startExamination',
          {
            requestType: 'form',
            data: {
              studentUnitExaminationId: state.id
            }
          }
        )
        const { questionJson, studentAnswerJson, ...res } = data
        const temp = questionJson || []
        temp.forEach((item: any) => {
          item.userAnswer = formatUserAnswers(item, studentAnswerJson)
        })
        console.log(temp)
        state.questionList = temp

        state.examDetail = { ...res } || {}

        calcTime()
      } catch {
        //
      }
    }

    /**
     * @description 计算考试时间剩余时间
     */
    const calcTime = () => {
      const examDetail = state.examDetail || {}
      const startTime = examDetail.startTime
      const nowTime = examDetail.now
      const timeMinutes = examDetail.timeMinutes || 0 // 测验时间
      // 返回秒
      const minu = dayjs(startTime).add(timeMinutes, 'minute').diff(dayjs(nowTime))
      if (minu <= 0) {
        state.visiableExam = true
      } else {
        state.time = Math.ceil(minu / 1000) * 1000
        setTimeout(() => {
          countDownRef.value?.start()
        }, 10)
      }
    }

    /**
     * @description 初始化用户答案
     */
    const formatUserAnswers = (item: any, userAnswer: any) => {
      // 判断是否有结果
      if (!userAnswer) return []
      const answers = JSON.parse(userAnswer) || []

      const questionItem = answers.find((child: any) => child.questionId === item.id)
      return questionItem ? questionItem.details : []
    }

    /**
     * @description 下一题 | 测试完成
     */
    const onNextQuestion = async () => {
      try {
        const questionList = state.questionList || []
        const userAnswerList: any = [] // 所有题目的答案

        let currentResult = false // 当前题目是否已经答题

        questionList.forEach((question: any, index: number) => {
          // 格式化所有题目的答案
          if (question.userAnswer && question.userAnswer.length > 0) {
            userAnswerList.push({
              questionId: question.id,
              details: question.userAnswer
            })
          }

          if (index === state.currentIndex) {
            currentResult = question.userAnswer && question.userAnswer.length > 0 ? true : false
          }
        })

        // 判断是否答题了
        if (!currentResult) {
          swipeRef.value?.next()
          return
        }

        state.nextStatus = true
        // 判断是否是最后一题
        if (state.questionList.length === state.currentIndex + 1) {
          // api-student/studentUnitExamination/completionExamination
          const { data } = await request.post(
            '/api-student/studentUnitExamination/completionExamination',
            {
              data: {
                answers: userAnswerList,
                studentUnitExaminationId: state.id
              }
            }
          )
          if (data.score >= state.examDetail.passScore) {
            state.resultStatusType = 'SUCCESS'
            state.resultInfo = {
              tips: '恭喜你，测验通过！',
              score: data.score,
              examName: data.unitExaminationName
            }
          } else {
            state.resultStatusType = 'FAIL'
            state.resultInfo = {
              tips: '本次测验不合格！',
              score: data.score,
              examName: data.unitExaminationName
            }
          }
          state.visiableResult = true
        } else {
          await request.post('/api-student/studentUnitExamination/submitAnswer', {
            hideLoading: true,
            data: {
              answers: userAnswerList,
              studentUnitExaminationId: state.id
            }
          })
          swipeRef.value?.next()
        }

        state.nextStatus = false
      } catch {
        //
        state.nextStatus = false
      }
    }

    onMounted(() => {
      getExamDetails()
    })

    return () => (
      <div class={styles.unitDetail}>
        <Cell center class={styles.unitSection}>
          {{
            title: () => <div class={styles.unitTitle}>{state.examDetail.unitExaminationName}</div>,
            label: () => (
              <div class={styles.unitCount}>
                <div class={styles.qNums}>
                  <Icon class={styles.icon} name={iconQuestionNums} />
                  题目数量{' '}
                  <span class={styles.num} style={{ paddingLeft: '6px' }}>
                    {state.currentIndex + 1}
                  </span>
                  /{state.examDetail.questionNum}
                </div>
                <div class={styles.qNums}>
                  <Icon class={styles.icon} name={iconCountDown} />
                  剩余时长：
                  <CountDown
                    ref={countDownRef}
                    v-model:time={state.time}
                    format={'mm:ss'}
                    autoStart={false}
                  />
                </div>
              </div>
            )
          }}
        </Cell>

        <Swipe
          loop={false}
          showIndicators={false}
          ref={swipeRef}
          duration={300}
          touchable={false}
          lazyRender
          // initialSwipe={state.currentIndex}
          onChange={(index: number) => {
            state.currentIndex = index
          }}
        >
          {state.questionList.map((item: any, index: number) => (
            // item.questionTypeCode === QuestionType.LINK && (
            //   <SwipeItem>
            //     <KeepLookQuestion v-model:value={item.userAnswer} data={item} index={index + 1} />
            //   </SwipeItem>
            // )
            <SwipeItem>
              {item.questionTypeCode === QuestionType.RADIO && (
                <ChoiceQuestion
                  v-model:value={item.userAnswer}
                  index={index + 1}
                  data={item}
                  type="radio"
                />
              )}
              {item.questionTypeCode === QuestionType.CHECKBOX && (
                <ChoiceQuestion
                  v-model:value={item.userAnswer}
                  index={index + 1}
                  data={item}
                  type="checkbox"
                />
              )}
              {item.questionTypeCode === QuestionType.SORT && <DragQuestion />}
              {item.questionTypeCode === QuestionType.LINK && (
                <KeepLookQuestion v-model:value={item.userAnswer} data={item} index={index + 1} />
              )}
              {item.questionTypeCode === QuestionType.PLAY && <PlayQuestion />}
            </SwipeItem>
          ))}
        </Swipe>

        <OSticky position="bottom" background="white">
          <div class={['btnGroup btnMore']}>
            {state.currentIndex > 0 && (
              <Button
                round
                block
                type="primary"
                plain
                onClick={() => {
                  swipeRef.value?.prev()
                }}
              >
                上一题
              </Button>
            )}
            <Button
              block
              round
              type="primary"
              onClick={onNextQuestion}
              loading={state.nextStatus}
              disabled={state.nextStatus}
            >
              {state.questionList.length === state.currentIndex + 1 ? '测试完成' : '下一题'}
            </Button>
            <Image
              src={iconButtonList}
              class={[styles.wapList, 'van-haptics-feedback']}
              onClick={() => (state.visiableAnswer = true)}
            />
          </div>
        </OSticky>

        {/* 题目集合 */}
        <ActionSheet v-model:show={state.visiableAnswer} title="题目列表" safeAreaInsetBottom>
          <AnswerList
            value={state.questionList}
            onSelect={(item: any) => {
              // 跳转，并且跳过动画
              swipeRef.value?.swipeTo(item, {
                immediate: true
              })
              state.visiableAnswer = false
            }}
          />
        </ActionSheet>

        <Popup
          v-model:show={state.visiableResult}
          closeOnClickOverlay={false}
          style={{ background: 'transparent', width: '96%' }}
        >
          <ResultFinish
            status={state.resultStatusType as any}
            result={state.resultInfo}
            confirmButtonText="我知道了"
            cancelButtonText="去练习"
            onClose={() => {
              state.visiableResult = false
              router.back()
            }}
            onConform={() => {
              console.log('Success')
              state.visiableResult = false
              router.back()
            }}
          />
        </Popup>

        <ODialog
          v-model:show={state.visiableSure}
          title="测验完成"
          message="确认本次测验的题目都完成了吗？\n提交后不可修改哦"
          messageAlign="left"
          showCancelButton
          cancelButtonText="再等等"
          confirmButtonText="确认完成"
        />

        <ODialog
          v-model:show={state.visiableExam}
          message="考试已结束"
          messageAlign="center"
          onConfirm={() => {
            state.visiableResult = true
          }}
        />
      </div>
    )
  }
})
